
function MapEditorHistory(mapEditor) {
	this.prevOverlays = [];
	this.presentOverlays = [];
	this.undoneOverlays = [];
	this.cycleCompvare = true;
	this.mapEditor = mapEditor;
}

MapEditorHistory.prototype.undo = function () {
	
	
	if (this.prevOverlays.length != 0) {
		for (var i = 0; i < this.mapEditor.overlays.length; i++) {
			this.mapEditor.overlays[i].setMap(null);
		}
		
		this.mapEditor.selectedShapes = [];
		this.mapEditor.overlays = [];
		this.undoneOverlaysPush();
		var prev_overlay = this.prevOverlays.pop();
		var draggableCB = document.querySelector("input[name=draggableCB]").checked = false;
		
		
		if (prev_overlay.length > 0) {
			for (var i = 0; i < prev_overlay.length; i++) {
				this.mapEditor.overlays[i] = prev_overlay[i];
				this.mapEditor.overlays[i].setMap(this.mapEditor.map);
				this.mapEditor.overlays[i].draggable = true;
			}
		}
	}
};

MapEditorHistory.prototype.redo = function () {

	if (this.undoneOverlays.length != 0) {
		if (this.undoneOverlays[this.undoneOverlays.length -1].length != 0) {
			var draggableCB = document.querySelector("input[name=draggableCB]").checked = false;
			this.mapEditor.selectedShapes = [];
			for (var i = 0; i < this.mapEditor.overlays.length; i++) {
				this.mapEditor.overlays[i].setMap(null);
			}

			this.mapEditor.overlays = [];
			var lastEntry = this.undoneOverlays[this.undoneOverlays.length - 1];
			for (var i = 0; i < lastEntry.length; i++) {
				this.mapEditor.overlays[i] = lastEntry[i];
				this.mapEditor.overlays[i].setMap(this.mapEditor.map);
				this.mapEditor.overlays[i].draggable = true;
			}

			var conditionPrevious = this.presentOverlays[0];
			if (conditionPrevious !== undefined) {
				if (conditionPrevious.length == 0) {
					this.prevOverlays.push(this.presentOverlays[0]);
				} else {
					var overlays_copy = [];
					for (var i = 0; i < this.presentOverlays[0].length; i++) {
						var clone_shape = ShapesUtil.cloneShape(this.presentOverlays[0][i]);
						if (ShapesUtil.overlayItemIsShape(clone_shape)) {
							this.mapEditor.addShapeListeners(clone_shape, null);
						} else {
							this.mapEditor.addMarkerListeners(clone_shape, null);
						}
						overlays_copy[i] = clone_shape;
					}
					this.prevOverlays.push(overlays_copy);
				}
			}
			this.presentOverlays = [];
			this.presentOverlays.push(this.undoneOverlays[this.undoneOverlays.length - 1]);
			this.undoneOverlays.pop();
		}
	}
}

/*
MapEditorHistory.prototype.clonePath = function (path) {
	var clone_path = [];

	for (var i = 0; i < path.length; i++) {
		var lati = path.getAt(i).lat();
		var lngi = path.getAt(i).lng();
		clone_lat_lng = {
			lat: lati,
			lng: lngi
		};
		clone_path.push(clone_lat_lng);
	}

	return clone_path;
}

MapEditorHistory.prototype.clonePolyline = function (polyline) {
	var geodesic = polyline.geodesic;
	var strokeColor = polyline.strokeColor;
	var strokeOpacity = polyline.strokeOpacity;
	var strokeWeight = polyline.strokeWeight;
	//var clone_path = clonePath(path);
	
	var clone_path = null;
	
	var JSONSave = typeof polyline.getPath !== "function";
	//console.log(JSONSave)
	if(JSONSave){
		var path = polyline.latLngs.j[0].j;
		clone_path = path;
		
	} else{
		var path = polyline.getPath();
		clone_path = this.clonePath(path);	
	}
	var clone_polyline = new google.maps.Polyline({
			geodesic: geodesic,
			strokeColor: strokeColor,
			strokeOpacity: strokeOpacity,
			strokeWeight: strokeWeight,
			path: clone_path,
			draggable: true,
			editable: false
		});

	clone_polyline.type = google.maps.drawing.OverlayType.POLYLINE;

	return clone_polyline;
}

MapEditorHistory.prototype.clonePolygon = function (polygon) {
	var geodesic = polygon.geodesic;
	var strokeColor = polygon.strokeColor;
	var strokeOpacity = polygon.strokeOpacity;
	var strokeWeight = polygon.strokeWeight;
	var fillColor = polygon.fillColor;
	var fillOpacity = polygon.fillOpacity;

	var clone_path = null;
	var JSONSave = typeof polygon.getPath !== "function";
	//console.log(JSONSave)
	if(JSONSave){
		var path = polygon.latLngs.j[0].j;
		clone_path = path;
		
	} else{
		var path = polygon.getPath();
		clone_path = this.clonePath(path);
		console.log(path);	
	}	
	var clone_polygon = new google.maps.Polygon({
			geodesic: geodesic,
			strokeColor: strokeColor,
			strokeOpacity: strokeOpacity,
			strokeWeight: strokeWeight,
			fillColor: fillColor,
			fillOpacity: fillOpacity,
			path: clone_path,
			draggable: true,
			editable: false
		});
	clone_polygon.type = google.maps.drawing.OverlayType.POLYGON;
	return clone_polygon;
}

MapEditorHistory.prototype.cloneRectangle = function (rect) {
	var strokeColor = rect.strokeColor;
	var strokeOpacity = rect.strokeOpacity;
	var strokeWeight = rect.strokeWeight;
	var fillColor = rect.fillColor;
	var fillOpacity = rect.fillOpacity;
	//var bounds = rect.getBounds();
	var type = rect.type;
	
	var bounds = null;
	
	var JSONSave = typeof rect.getBounds !== "function";
	
	if(JSONSave){
		var north = rect.bounds.north;
		var south = rect.bounds.south;
		var east = rect.bounds.east;
		var west = rect.bounds.west;
		if (!isNaN(north) && !isNaN(south) && !isNaN(west) && !isNaN(east)) {
			var NE = new google.maps.LatLng(north, east);
			var SW = new google.maps.LatLng(south, west);
			bounds = new google.maps.LatLngBounds(SW, NE);
		}		
	} else {
		bounds = rect.getBounds();
	}
	var clone_rect = new google.maps.Rectangle({
			strokeColor: strokeColor,
			strokeOpacity: strokeOpacity,
			strokeWeight: strokeWeight,
			fillColor: fillColor,
			fillOpacity: fillOpacity,
			bounds: bounds,
			draggable: true,
			editable: false,
			type: type
		});

	clone_rect.type = google.maps.drawing.OverlayType.RECTANGLE;
	return clone_rect;
}

MapEditorHistory.prototype.cloneCircle = function (circ) {
	var strokeColor = circ.strokeColor;
	var strokeOpacity = circ.strokeOpacity;
	var strokeWeight = circ.strokeWeight;
	var fillColor = circ.fillColor;
	var fillOpacity = circ.fillOpacity;
	var center = circ.center;
	var radius = circ.radius;
	
	var clone_circ = new google.maps.Circle({
			strokeColor: strokeColor,
			strokeOpacity: strokeOpacity,
			strokeWeight: strokeWeight,
			fillColor: fillColor,
			fillOpacity: fillOpacity,
			center: center,
			radius: radius,
			draggable: true,
			editable: false
		});

	clone_circ.type = google.maps.drawing.OverlayType.CIRCLE;

	return clone_circ;
}

MapEditorHistory.prototype.cloneMarker = function (marker) {
	var anchorPoint = marker.anchorPoint;
	var JSONSave = typeof marker.getPosition !== "function";
	if(JSONSave){
		var position = marker.position;
	} else{
		var position = marker.getPosition();
	}
	var clone_marker = new google.maps.Marker({
			anchorPoint: anchorPoint,
			position: position,
			clickable: true,
			draggable: true,
			editable: false
		})
		clone_marker.type = google.maps.drawing.OverlayType.MARKER;

	return clone_marker;
}

MapEditorHistory.prototype.cloneShape = function (shape) {
	if (shape.type === google.maps.drawing.OverlayType.POLYLINE) {
		var clone_polyline = this.clonePolyline(shape);
		return clone_polyline;
	} else if (shape.type === google.maps.drawing.OverlayType.POLYGON) {
		var clone_polygon = this.clonePolygon(shape);
		return clone_polygon;
	} else if (shape.type === google.maps.drawing.OverlayType.RECTANGLE) {
		var clone_rect = this.cloneRectangle(shape);
		return clone_rect;

	} else if (shape.type === google.maps.drawing.OverlayType.CIRCLE) {
		var clone_circ = this.cloneCircle(shape);
		return clone_circ;

	} else {
		var clone_marker = this.cloneMarker(shape);
		return clone_marker;
	}
}

MapEditorHistory.prototype.overlayItemIsShape = function (overlay_item) {
	var type = overlay_item.type;

	is_shape = (type === google.maps.drawing.OverlayType.POLYLINE)
	 || (type === google.maps.drawing.OverlayType.POLYGON)
	 || (type === google.maps.drawing.OverlayType.RECTANGLE)
	 || (type === google.maps.drawing.OverlayType.CIRCLE);

	return is_shape;
}

*/

MapEditorHistory.prototype.historyOverlayPush = function () {
	
	
	if (this.cycleCompvare) {
		
		var overlays_copy = [];
		for (var i = 0; i < this.mapEditor.overlays.length; i++) {
			var clone_shape = ShapesUtil.cloneShape(this.mapEditor.overlays[i]);
			if (ShapesUtil.overlayItemIsShape(clone_shape)) {
				this.mapEditor.addShapeListeners(clone_shape, null); // don't have an overlay event!
			} else {
				this.mapEditor.addMarkerListeners(clone_shape, null); // don't have an overlay event!
			}
			overlays_copy[i] = clone_shape;
		}
		this.undoneOverlays = [];
		this.prevOverlays.push(overlays_copy);
	}

	this.cycleCompvare = false;
}

MapEditorHistory.prototype.presentOverlayPush = function () {
//	console.log("presentOverlayPush");
	
	this.presentOverlays = [];
	var overlays_copy = [];
	for (var i = 0; i < this.mapEditor.overlays.length; i++) {
		var clone_shape = ShapesUtil.cloneShape(this.mapEditor.overlays[i]);
		if (ShapesUtil.overlayItemIsShape(clone_shape)) {
			this.mapEditor.addShapeListeners(clone_shape, null); // don't have an overlay event!
		} else {
			this.mapEditor.addMarkerListeners(clone_shape, null); // don't have an overlay event!
		}
		overlays_copy[i] = clone_shape;
	}
	this.presentOverlays.push(overlays_copy);
	this.cycleCompvare = true;

	
}

MapEditorHistory.prototype.undoneOverlaysPush = function () {
	
	var conditionUndone = this.presentOverlays[this.presentOverlays.length - 1] !== undefined;

	if (conditionUndone) {
		var overlays_copy = [];
		for (var i = 0; i < this.presentOverlays[0].length; i++) {
			var clone_shape = ShapesUtil.cloneShape(this.presentOverlays[0][i]);
			if (ShapesUtil.overlayItemIsShape(clone_shape)) {
				this.mapEditor.addShapeListeners(clone_shape, null); // don't have an overlay event!
			} else {
				this.mapEditor.addMarkerListeners(clone_shape, null); // don't have an overlay event!
			}
			overlays_copy[i] = clone_shape;
		}
		this.undoneOverlays.push(overlays_copy);
	}

	var conditionPresent = this.prevOverlays[this.prevOverlays.length - 1] !== undefined;

	if (conditionPresent) {
		this.presentOverlays = [];
		var overlays_copy = [];
		for (var i = 0; i < this.prevOverlays[this.prevOverlays.length - 1].length; i++) {
			var clone_shape = cloneShape(this.prevOverlays[this.prevOverlays.length - 1][i]);
			if (ShapesUtil.overlayItemIsShape(clone_shape)) {
				this.mapEditor.addShapeListeners(clone_shape, null); // don't have an overlay event!
			} else {
				this.mapEditor.addMarkerListeners(clone_shape, null); // don't have an overlay event!
			}
			overlays_copy[i] = clone_shape;
		}
		this.presentOverlays.push(overlays_copy);
	}
}

MapEditorHistory.prototype.undoneOverlaysPush = function () {

	var conditionUndone = this.presentOverlays[this.presentOverlays.length - 1] !== undefined;

	if (conditionUndone) {
		var overlays_copy = [];
		for (var i = 0; i < this.presentOverlays[0].length; i++) {
			var clone_shape = ShapesUtil.cloneShape(this.presentOverlays[0][i]);
			if (ShapesUtil.overlayItemIsShape(clone_shape)) {
				this.mapEditor.addShapeListeners(clone_shape, null); // don't have an overlay event!
			} else {
				this.mapEditor.addMarkerListeners(clone_shape, null); // don't have an overlay event!
			}
			overlays_copy[i] = clone_shape;
		}
		this.undoneOverlays.push(overlays_copy);
	}

	var conditionPresent = this.prevOverlays[this.prevOverlays.length - 1] !== undefined;

	if (conditionPresent) {
		this.presentOverlays = [];
		var overlays_copy = [];
		for (var i = 0; i < this.prevOverlays[this.prevOverlays.length - 1].length; i++) {
			var clone_shape = ShapesUtil.cloneShape(this.prevOverlays[this.prevOverlays.length - 1][i]);
			if (ShapesUtil.overlayItemIsShape(clone_shape)) {
				this.mapEditor.addShapeListeners(clone_shape, null); // don't have an overlay event!
			} else {
				this.mapEditor.addMarkerListeners(clone_shape, null); // don't have an overlay event!
			}
			overlays_copy[i] = clone_shape;
		}
		this.presentOverlays.push(overlays_copy);
	}
}